# NEC_empf_1.py
# erstellt und getestet am 13.11.2014
# Funktioniert mit majority-Fernbedienung und KeuwlSoft-App

from time import ticks_us, sleep_ms, ticks_diff
from machine import Pin
from sys import exit

signal_in = Pin(25, Pin.IN) # für Empfangssignal (Aufnahme); IR-Receiver
# Toleranzen können auch kleiner sein z. B. +/- 500 us
leading_burst_0 = 9_000 # us
leading_burst_min = 8_000
leading_burst_max = 10_000
leading_space = 4_500
leading_space_min = 3_500
leading_space_max = 5_500

T1 = 1125 # us

address = [0,0,0,0,0,0,0,0]
address_inv = [0,0,0,0,0,0,0,0]
command = [0,0,0,0,0,0,0,0]
command_inv = [0,0,0,0,0,0,0,0]


###################### Funktionen ########################

def wait_for_high():
    while signal_in.value() == 0:
        pass
    T = ticks_us()
    return T


def wait_for_low():
    while signal_in.value() == 1:
        pass
    T = ticks_us()
    return T


def wait_for_leading_pulse(): # Zu Beginn liegt Low-Signal vor
    Ta = wait_for_high()
    Te = wait_for_low()
    leading_burst = ticks_diff(Te, Ta)
    print('lburst', leading_burst)
    if leading_burst_min < leading_burst < leading_burst_max:
        # Jetzt liegt High-Signal vor
        Ta = wait_for_low()
        Te = wait_for_high()
        leading_space = ticks_diff(Te, Ta)
        print('lspace', leading_space)
        if leading_space_min < leading_space < leading_space_max:
            return True  # Jetzt liegt High-Signal vor
        else:
            return False
    else:
        return False

def get_byte(byte_list): # zu Beginn High-Signal
    for i in range(8):
        Ta = wait_for_low()
        Te = wait_for_high() # Jetzt am Ende des Bits
        deltaT = ticks_diff(Te, Ta)
        if deltaT < T1:
            byte_list[i] = 0
        else:
            byte_list[i] = 1
    return byte_list     # Am Ende liegt High-Signal vor

def inv_test(list1, list2):
    result = True
    for i in range(8):
        result = result and (list1[i]  != list2[i])
    return result

def bit_list_to_nr(l): # Bit-Liste in Zahl konvertieren (LSB ... LSB)
    n = len(l)
    value = l[n-1]
    for i in range(1,n):
        value = value * 2 + l[n-1-i]
    return value


############################# Hauptprogramm #################################

print('Programm gestartet... (Abbrechen mit Strg-C)')

while True:
    # Empfang eines NEC-Signals
    # Zu Beginn liegt Low-Signal vor
    if wait_for_leading_pulse() == False:
        print('leading pulse error')
        exit()
    # Jetzt liegt High-Signal vor   
    address = get_byte(address) # Liste
    address_inv = get_byte(address_inv) # Liste
    command = get_byte(command) # Liste
    command_inv = get_byte(command_inv) # Liste
    # Jetzt liegt High-Signal vor (Anfang vom Ende-Burst)
    Ta = ticks_us()
    Te = wait_for_low() # Ende des Bursts und Ende des gesamten NEC-Signals
    end_burst = ticks_diff(Te, Ta)
    # Jetzt liegt wieder ein Low-Signal vor, bis ein neues NEC-Signal empfangen wird

    # Ausgabe der Werte:
    print('End-Burst', end_burst)
    print('Adresse:', address, '->', bit_list_to_nr(address))
    print('Komplement der Adresse:', address_inv)
    print('Adress-Test:', inv_test(address, address_inv))
    print('Befehl:', command, '->', bit_list_to_nr(command))
    print('Komplement des Befehls:', command_inv)
    print('Befehls-Test:', inv_test(command, command_inv))    
    print()
    sleep_ms(200) # falls Finger nicht schnell genung von der Taste weg...
    # Jetzt bereit für ein nächstes NEC-Signal


