# RC5_empf_1a.py: Zeichenketten über IR-Modul im Manchestercode (mit 36 kHz Trägersignal) empfangen
# Achtung: Das vom IR-Modul abgegebene Signal ist invertiert und muss z. B. mittels einer Transistor-Schaltung invertiert werden.
# Präambel: zwei 1-Bits; Payload: 1 Byte, d. h. 8 Bits; T ≈ 900 us (real)
# Sender sollte zwischen zwei Zeichen ca 30 ms warten. 
# getestet mit RC5_send_2c.py am 23.09.2024 


from time import sleep, sleep_us, sleep_ms
from machine import Pin, PWM

signal = Pin(25, Pin.IN) # für Empfangssignal

def wait_for_start_bit():
    while not signal.value():
        pass

def get_bit_list(n, t): # n = Anzahl der Bits, t = Zeit für 1 (ganzes) Bit
    global signal
    l = []
    for i in range(n):
        l.append(signal.value())
        sleep_us(t)
    return l

def bit_list_to_nr(l): # Zahl
    n = len(l)
    value = l[0]
    for i in range(1, n):
        value = (value << 1) + l[i] # value = value * 2 + l[i]
    return value    

'''
# Test für bit_list_to_nr:          
bit_liste = [1, 1, 1, 1, 1, 1, 0, 1]
print(bit_list_to_nr(bit_liste))
'''

# Hauptprogramm
T = 875 # Halbe Bitzeit in us (875 - 885 ganz gut)
T_2 = T * 2
T_4_2 = 4 * T + T // 2 # s. u.

nr_of_bits = 8 # Hier werden also Bytes übertragen

print('Warte auf Botschaften...')

# Einzelne Bytes empfangen und auf dem Terminal ausgeben
while True:
    wait_for_start_bit()
    sleep_us(T_4_2) # jetzt in der Mitte der 2. Hälfte des 1. Payload-Bits (nr_of_startbits = 2)
    bit_list = get_bit_list(nr_of_bits, T_2)
    # print(bit_list) # zu Testzwecken
    value = bit_list_to_nr(bit_list)
    # print(value) # zu Testzwecken
    if nr_of_bits == 8:
        print(chr(value), end = '')
    sleep_us(T) # jetzt hinter dem letzten Bit
    
    
    
    
    


