# RC5_empf_3d.py
# Grund-Idee: Liste von Zeitintervallen zwischen Flanken aufnehmen und anschließend auswerten (mehr dazu unten...)
# getestet am 11.11.2024

# Signal von RC-5-Fernbedienung empfangen (ggf. mit Keuwl-Remote-App)
# und die Werte für Toggle-Bit, das Gerät sowie den Befehl ausgeben.
# Dabei soll ein Toleranzwert (in %) für die kurzen und langen Zeitintervalle einstellbar sein.

# Strategie:
# Zunächst Länge der High- und Low-Phasen (in us) in einer Liste speichern
# Anschließend aus dieser Liste eine Liste mit den Zuständen der Halb-Bits erstellen; diese kann
# man auch als Signal-Diagramm deuten.
# Daraus kann man schließlich eine Bit-liste erstellen, aus der wie in RC5_Fernbed_empf_1.py
# die gesuchten Daten abgeleitet werden können.

# ticks_us scheint beim ersten Aufruf einen etwas verspäteten Zeitwert zu liefern.
# Deswegen wird bei der Funktion record_deltaTs zu Beginn mit "dummy = ticks_us()"
# ein (erster) nicht benutzter (ungenauer) Zeitwert abgerufen.


from time import ticks_us, sleep_ms, ticks_diff
from machine import Pin
from sys import exit

T = 890 # Halbe Bitzeit
signal_in = Pin(25, Pin.IN) # für Empfangssignal (Aufnahme); IR-Receiver
T_max = 30_000 # maximale Signallänge (in us) bei RC-5

tolerance = 25
# in Prozent; funktioniert bei meiner DUAL-Fernbedienungen auch schon mit 10%,
# bei der ungenaueren KeuwlSoft-App mit 25%

###################### Funktionen ########################

def wait_for_high():
    global T_Start
    while (signal_in.value() == 0) and (ticks_us() - T_Start < T_max):  # solange keine Zeitüberschreitung vorliegt
        pass
    Te = ticks_us()
    return Te


def wait_for_low():
    while signal_in.value() == 1:
        pass
    Te = ticks_us()
    return Te


def record_deltaTs():
    dummy = ticks_us() # dadurch wird der erste Zeitwert etwas besser
    global T_Start 
    deltaT_list = [890]
    # Auf erste positive Flanke warten; Startzeit T_Start merken:
    while signal_in.value() == 0:
        pass
    T_Start = ticks_us() # Beginn des ersten High-Signals
    Ta = T_Start

    # Solange seit dem Start T_max nicht überschritten werden...
    while (ticks_diff(ticks_us(), T_Start)) < T_max: 
        Te = wait_for_low()  # Zeit beim Ende der (nächsten) High-Phase
        deltaT = Te - Ta # Dauer des High-Zustands
        deltaT_list.append(deltaT)
        Ta = Te
        Te = wait_for_high() # Zeit beim Ende der (nächsten) Low-Phase; liefert bei Zeitüberschreitung die Restzeit bis T_max
        deltaT = Te - Ta # Dauer des Low-Zustands
        deltaT_list.append(deltaT)
        Ta = Te
    return deltaT_list[0:-1] # Restzeit aus der Liste entfernen

def DeltaTs_to_HB(deltaT_list, tolerance):
    # Halb-Bit-Liste aus deltaT_list ableiten
    HB_list =[False] # Erstes Halb-Bit (der Präambel) ist immer 0/False und nicht zu detektieren.
    HB_count = 0
    min_1 = int(T*(1-tolerance/100))
    max_1 = int(T*(1+tolerance/100))
    min_2 = T + min_1
    max_2 = T + max_1
    for deltaT_count in range(len(deltaT_list)): 
        if min_1 < deltaT_list[deltaT_count] < max_1: # kurzes DeltaT; Toleranz (s. o.); ggf. Fehlermeldung, wenn außerhalb der Toleranz
            HB_list.append(not HB_list[HB_count])
            HB_count += 1
        elif min_2 < deltaT_list[deltaT_count] < max_2: # langes DeltaT
            HB_list.append(HB_list[HB_count]) # später s. o.
            HB_count += 1
            HB_list.append(not HB_list[HB_count]) # später s. o. 
            HB_count += 1
        else:
            print('Fehler beim Umwandeln in HB-Liste beim', HB_count+1, '-ten DeltaT: Kein RC-5-Signal?')
            exit()
    if len(HB_list) % 2 == 1:
        HB_list = HB_list[0:-1] # je nachdem, ob am Ende ein True steht oder nicht
    return HB_list
  

def HB_to_B_list(HB): # Halb-Bit-Liste in Bit-Liste umwandeln
    B = []
    nr = len(HB) / 2 
    for i in range(nr):
        Bit_pair = HB[(2*i):(2*i+2)]
        if Bit_pair == [False, True]:
            B.append(1)
        elif Bit_pair == [True, False]: 
            B.append(0)
        else:
            print('Fehler beim Umwandeln von HB-Liste in B-Liste: Kein RC-5-Signal?')
            exit()
    return B


def bit_list_to_nr(l): # Bit-Liste in Zahl konvertieren
    n = len(l)
    value = l[0]
    for i in range(1, n):
        value = (value << 1) + l[i] # value = value * 2 + l[i]
    return value


############################# Hauptprogramm #################################

print('Programm gestartet... (Abbrechen mit Strg-C)')

while True:
    print('Jetzt RC-5-Signal mit Fernbedienung (ggf. Remote-App von KeuwlSoft) senden!')
    # delta_T-Liste erstellen
    deltaT_list = record_deltaTs()
    print('deltaT_list:', deltaT_list)
    HB_list = DeltaTs_to_HB(deltaT_list, tolerance)
    print('Halb-Bit-Liste:', HB_list)
    print('Ergebnis:')

    # Bit-Liste:
    bit_list = HB_to_B_list(HB_list[4:]) # die 2 Präambel-Bits abschneiden
    # RC-5-Daten ausgeben:
    print('Toggle-Bit: ', bit_list[0])
    print('Gerät: ', bit_list_to_nr(bit_list[1:6]))
    print('Befehl: ', bit_list_to_nr(bit_list[6:12]))
    print()
  
    offset = 50
    sleep_ms(150) # warten für langsame Bediener





