# RC5_send_2d: (mehrere) Zeichenketten (mit einer IR-LED an Pin 25) im PWM-Manchestercode senden (36 kHz)
# vgl. auch RC5_send_2c.py
# getestet mit rc5_empf_2a.py


from time import sleep, sleep_us, sleep_ms
from machine import Pin, PWM
led = Pin(25, Pin.OUT)
pwm_led = PWM(led, freq=36_000, duty_u16=0) # 0 <= duty_u16 < 2^^16=65536


T = 870 # Halbe Bitzeit in us (ergibt real ca. 900 us  +/- 3% real)
# T = 7000 # funktioniert für 700 <= T <= 7000 mit rc5_empf_2a.py

def send_symbol(b): # b ist True (=1) oder False (=0)
    if b:
        pwm_led.duty_u16(0)  
        sleep_us(T)
        pwm_led.duty_u16(32768) 
        sleep_us(T)
    else:
        pwm_led.duty_u16(32768)  
        sleep_us(T)
        pwm_led.duty_u16(0)
        sleep_us(T)
    
def to_bitlist(anzahl_bits, n):   # anzahl_bits: Anzahl der Bits (bei der Übertragung eines Bytes: 8); n: Zahl (int), z. B. ASCII-Code eines Zeichens
    bl = []
    for i in range(anzahl_bits):
        j = anzahl_bits - 1 - i
        if n & (1 << j) : # n & (1 << j) > 0 => True
            bl.append(1)
        else:
            bl.append(0)
    return bl

def send_bitlist(b_l): # sendet die Signalfolge für die Bit-Liste bl
    for i in b_l:
        send_symbol(i)

print('Geben Sie jetzt die zu sendenden Zeichenketten ein! Abbruch mit Strg-C')
print()

while True:
    # Eingabe einer Zeichenkette:
    zk = input('Text: ') # liefert Warnung bei non-ASCII-Zeichen
    zk= zk + '\n' # Steuerzeichen für neue Zeile anhängen
    print('Zeichenkette wird gesendet...')
    print()
    # Zeichenkette senden...
    for c in zk:
        pre_list = [1,1] # Präambel
        ascii_code = ord(c)
        bit_list = to_bitlist(8, ascii_code) # Bitliste    
        # Präambel senden:
        send_bitlist(pre_list)
        # Bitliste für Zeichen senden:
        send_bitlist(bit_list)
        pwm_led.duty_u16(0) # mit Low-Signal beenden
        sleep_ms(40)  # 20 zu wenig

